package com.logicbig.example;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.Query;
import java.time.LocalDate;
import java.util.List;

public class ExampleMain {
    private static EntityManagerFactory entityManagerFactory =
            Persistence.createEntityManagerFactory("example-unit");

    public static void main(String[] args) {
        try {
            persistEmployees();
            System.out.println("-- Finding Employee by name --");
            findEmployeeByName();
            System.out.println("-- Finding Employee by salary --");
            findEmployeeBySalary();
            System.out.println("-- Finding Employee by dept --");
            findEmployeeByDept();
            System.out.println("-- Finding Employee by join date --");
            findEmployeeByJoinDate();
            System.out.println("-- Finding full time Employee --");
            findFullTimeEmployee();
        } finally {
            entityManagerFactory.close();
        }
    }

    public static void persistEmployees() {
        Employee employee1 = Employee.create("Diana", Dept.IT, 3000, LocalDate.of(1999, 11, 15), true);
        Employee employee2 = Employee.create("Rose", Dept.ADMIN, 4000, LocalDate.of(2011, 5, 1), false);
        Employee employee3 = Employee.create("Denise", Dept.IT, 1500, LocalDate.of(2006, 1, 10), true);
        Employee employee4 = Employee.create("Mike", Dept.SALE, 2000, LocalDate.of(2015, 8, 20), false);
        EntityManager em = entityManagerFactory.createEntityManager();
        em.getTransaction().begin();
        em.persist(employee1);
        em.persist(employee2);
        em.persist(employee3);
        em.persist(employee4);
        em.getTransaction().commit();
        em.close();
    }

    private static void findEmployeeByName() {
        EntityManager em = entityManagerFactory.createEntityManager();
        Query query = em.createQuery("SELECT e FROM Employee e WHERE e.name = 'Rose'");
        List<Employee> resultList = query.getResultList();
        resultList.forEach(System.out::println);
        em.close();
    }

    private static void findEmployeeBySalary() {
        EntityManager em = entityManagerFactory.createEntityManager();
        Query query = em.createQuery("SELECT e FROM Employee e WHERE e.salary > 2000L");
        List<Employee> resultList = query.getResultList();
        resultList.forEach(System.out::println);
        em.close();
    }

    private static void findEmployeeByDept() {
        EntityManager em = entityManagerFactory.createEntityManager();
        Query query = em.createQuery("SELECT e FROM Employee e WHERE e.dept = com.logicbig.example.Dept.IT");
        List<Employee> resultList = query.getResultList();
        resultList.forEach(System.out::println);
        em.close();
    }

    private static void findEmployeeByJoinDate() {
        EntityManager em = entityManagerFactory.createEntityManager();
        Query query = em.createQuery("SELECT e FROM Employee e WHERE e.joinDate > '2010-05-01'");
        List<Employee> resultList = query.getResultList();
        resultList.forEach(System.out::println);
        em.close();
    }

    private static void findFullTimeEmployee() {
        EntityManager em = entityManagerFactory.createEntityManager();
        Query query = em.createQuery("SELECT e FROM Employee e WHERE e.fullTime = TRUE");
        List<Employee> resultList = query.getResultList();
        resultList.forEach(System.out::println);
        em.close();
    }
}